==> em_tasks-0.1.65-py2.py3-none-any.whl: em_tasks/spree/product_util.py <==
import os
import time
import json
import csv
import tempfile
import uuid
import pdb

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.spree.api import SpreeApi


class ProductUtil():
  def __init__(self, endpoint, api_key, api_version='v1'):
    self.spree_api = SpreeApi(endpoint, api_key, api_version)
    # self.shop = self.spree_api.get_shop()
    # url = self.shop['url'] if self.shop['url'].startswith('http') else 'https://{}'.format(self.shop['url'])
    # parsed = urlparse(self.shop['url'])
    # self.shop_domain = parsed.hostname if parsed.hostname else 'everymarket.com'

  def get_amz_products(self, marketplace='us'):
    for inv in self.get_products('Amazon', marketplace):
      yield inv

  def get_ebay_products(self, marketplace='us'):
    for inv in self.get_products('Ebay', marketplace):
      yield inv

  def get_nandansons_products(self):
    for inv in self.get_products('nandansons'):
      yield inv

  def get_rakuten_products(self):
    for inv in self.get_products('Rakuten'):
      yield inv

  def get_st11_products(self):
    for inv in self.get_products('11ST'):
      yield inv

  def get_apo_health_products(self):
    for inv in self.get_products('ApoHealth'):
      yield inv

  def get_lotteon_products(self):
    for inv in self.get_products('lotteon'):
      yield inv

  def get_wemakeprice_products(self):
    for inv in self.get_products('WEMAKEPRICE'):
      yield inv

  def get_naver_products(self):
    for inv in self.get_products('Naver'):
      yield inv

  def get_dangdang_products(self):
    for inv in self.get_products('Dangdang'):
      yield inv

  def get_coupang_products(self):
    for inv in self.get_products('Coupang'):
      yield inv

  def get_faire_products(self):
    for inv in self.get_products('faire'):
      yield inv

  def get_www_books_com_tw_products(self):
    for inv in self.get_products('www_books_com_tw'):
      yield inv

  def get_products(self, src, marketplace=None):
    source = None
    if src == 'Amazon':
      source = 'AMZ_{}'.format(marketplace.upper())
    elif src == 'Ebay':
      source = 'Ebay_{}'.format(marketplace.upper())
    elif src in [
        'Dangdang', 'Rakuten', 'Coupang', 'Naver', 'www_books_com_tw', "WEMAKEPRICE", "11ST",
        "nandansons", "lotteon", "ApoHealth"
        ]:
      source = src
    else:
      source = src

    if source is None:
      return

    # _, output_path = tempfile.mkstemp('.csv', '{}-'.format(source))
    temp_dir = tempfile.gettempdir()
    output_path = os.path.join(temp_dir, '{}-{}.csv'.format(source, str(uuid.uuid4())))
    try:
      logger.info("[InventoryPath] %s", output_path)
      self.spree_api.download_inventory(source, output_path)

      if not os.path.isfile(output_path):
        return

      try:
        manual_products = {pid:None for pid in self.get_manual_products()}
      except:
        manual_products = {}

      with open(output_path, encoding='utf-8', errors='ignore') as fh:
        reader = csv.reader(fh)
        headers = next(reader)
        headers_mapping = {
          'ProductID': 'product_id',
          'Source': 'source',
          'SourceProductID': 'source_product_id',
          'Handle': 'handle',
          'Variants': 'variants',
          'InStock': 'in_stock'
        }
        formatted_headers = []
        for header in headers:
          if header not in headers_mapping:
            continue

          formatted_headers.append(headers_mapping[header])

        for row in reader:
          if not row:
            continue

          record = dict()
          try:
            for idx, header in enumerate(formatted_headers):
              if header == 'variants':
                record[header] = json.loads(row[idx])
              else:
                record[header] = row[idx]
          except Exception as e:
            logger.info('[InventoryRow] %s', row)
            logger.exception(e)
            continue

          if not record['product_id'] or not record['source'] or not record['source_product_id']:
            continue

          if record['product_id'] in manual_products:
            continue

          yield record
    finally:
      if os.path.isfile(output_path):
        os.remove(output_path)

  def get_manual_products(self):
    result = None
    try:
      result = self.spree_api.get_manual_products()
    except Exception as e:
      logger.exception(e)

    if not result or 'error' in result or 'errors' in result:
      return []

    return result.get('products', [])

  def set_products_offer(self, prods, pool=None):
    store_offers = dict()
    for prod_id, prod in prods.items():
      store_offer = {'handle': prod['handle'], 'product_id': prod_id, 'offers': {}}
      variants = prod.get('variants', None)
      if not variants:
        continue

      offer = prod.get('offer', None)
      # Offer service encountered errors
      if isinstance(offer, bool) and not offer:
        continue

      # Product is not available on offer service
      if offer is None:
        continue

      # Product with multiple variants should supply each variant's offer
      if len(variants) > 1:
        if not isinstance(offer, dict):
          continue
      else:
        v = variants[0]
        if v['variant_id'] not in offer:
          offer = {v['variant_id']: offer}

      for variant in variants:
        v_offer = offer.get(variant['variant_id'], None)
        if not v_offer:
          continue

        price = round(float(v_offer['price']), 2)
        quantity = round(float(v_offer['quantity']), 2)
        store_offer['offers'][variant['variant_id']] = {
          'product_id': prod_id,
          'variant_id': variant['variant_id'],
          'price': price,
          'quantity': quantity,
        }
        logger.debug(
          '[InventoryUpdate] ProductId: %s, VariantId: %s, Price: %s, Quantity: %s',
          prod_id, variant['variant_id'], price, quantity)

      store_offers[prod_id] = store_offer

    resp = self.spree_api.set_offers(store_offers)
    logger.info('[InventoryUpdated] %s', resp)

    return store_offers

  def set_inventory(self, stock_items):
    succeed = True
    for stock_item in stock_items:
      max_retries = 3
      while max_retries > 0:
        try:
          resp = self.spree_api.update_stock(
            stock_item['id'], stock_item['quantity'], stock_item['stock_location_id'])
          if resp.status_code == 422:
            logger.debug(
              '[StockUpdateMsg] VariantId: %s, Stock: %s, Quantity: %s, Response: %s',
              stock_item['variant_id'], stock_item['id'], stock_item['quantity'], resp.text)
            max_retries -= 1
            time.sleep(1)

            continue

          try:
            stock = resp.json()
            if 'error' in stock or 'errors' in stock:
              succeed = False
              logger.warning(
                '[StockUpdateError] VariantId: %s, Stock: %s, Error: %s',
                stock_item['variant_id'], stock_item['id'],
                stock.get('error', stock.get('errors')))
            else:
              logger.info(
                '[StockUpdated] VariantId: %s, Stock: %s, Quantity: %s',
                stock_item['variant_id'], stock_item['id'], stock_item['quantity'])
          except Exception as e:
            succeed = False
            logger.exception(e)
            logger.warning('[StockUpdateInvalidResponse] %s', resp.text)

          break
        except Exception as e:
          logger.exception(e)
          max_retries -= 1
          time.sleep(1)

    return succeed

  def set_price(self, product_id, variant_id, price, cost=None):
    succeed = True
    max_retries = 3
    while max_retries > 0:
      try:
        if cost:
          resp = self.spree_api.update_variant(product_id, variant_id, price=price, cost_price=cost)
        else:
          resp = self.spree_api.update_variant(product_id, variant_id, price=price)
        if resp.status_code == 422:
          logger.debug(
            '[PriceUpdateMsg] ProductID: %s, VariantId: %s, Price: %s, Response: %s',
            product_id, variant_id, price, resp.text)
          max_retries -= 1
          time.sleep(1)

          continue

        sku = ''
        try:
          variant = resp.json()
          if 'error' in variant or 'errors' in variant:
            succeed = False
            logger.warning(
              '[PriceUpdateError] ProductId: %s, VariantId: %s, Error: %s',
              product_id, variant_id, variant.get('error', variant.get('errors')))
          elif 'sku' in variant and variant['sku']:
            sku = variant['sku']
            logger.info(
              '[PriceUpdated] ProductId: %s, VariantId: %s, Price: %s, SKU: %s',
              product_id, variant_id, price, sku)
          else:
            logger.info('[ResponseWithoutSKU] %s', resp.text)
        except:
          succeed = False
          logger.warning('[PriceUpdateInvalidResponse] %s', resp.text)

        break
      except Exception as e:
        succeed = False
        logger.warning(
          '[PriceUpdateError] ProductId: %s, VariantId: %s, Price: %s',
          product_id, variant_id, price)
        logger.exception(e)

    return succeed

  def download_inventory(self, source, output_path):
    self.spree_api.download_inventory(source, output_path)

    if not os.path.isfile(output_path):
      return

    with open(output_path, encoding='utf-8', errors='ignore') as fh:
      reader = csv.reader(fh)
      headers = next(reader)
      headers_mapping = {
        'ProductID': 'product_id',
        'Source': 'source',
        'SourceProductID': 'source_product_id',
        'Handle': 'handle',
        'Variants': 'variants'
      }
      formatted_headers = []
      for header in headers:
        if header not in headers_mapping:
          continue

        formatted_headers.append(headers_mapping[header])

      for row in reader:
        if not row:
          continue

        record = dict()
        try:
          for idx, header in enumerate(formatted_headers):
            if header == 'variants':
              record[header] = json.loads(row[idx])
            else:
              record[header] = row[idx]
        except Exception as e:
          logger.info('[InventoryRow] %s', row)
          logger.exception(e)
          continue

        yield record


if __name__ == "__main__":
  pass

==> em_tasks-0.1.66-py2.py3-none-any.whl: em_tasks/spree/product_util.py <==
import os
import time
import json
import csv
import tempfile
import uuid
import pdb

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.spree.api import SpreeApi


class ProductUtil():
  def __init__(self, endpoint, api_key, api_version='v1'):
    self.spree_api = SpreeApi(endpoint, api_key, api_version)
    # self.shop = self.spree_api.get_shop()
    # url = self.shop['url'] if self.shop['url'].startswith('http') else 'https://{}'.format(self.shop['url'])
    # parsed = urlparse(self.shop['url'])
    # self.shop_domain = parsed.hostname if parsed.hostname else 'everymarket.com'
  def get_daisomall_products(self):
    for inv in self.get_products('Daisomall'):
      yield inv

  def get_trendyol_products(self, marketplace='us'):
    for inv in self.get_products('Trendyol', marketplace):
      yield inv

  def get_amz_products(self, marketplace='us'):
    for inv in self.get_products('Amazon', marketplace):
      yield inv

  def get_emp_products(self):
    for inv in self.get_products('EMP'):
      yield inv

  def get_boots_products(self):
    for inv in self.get_products('Boots'):
      yield inv

  def get_ebay_products(self, marketplace='us'):
    for inv in self.get_products('Ebay', marketplace):
      yield inv

  def get_nandansons_products(self):
    for inv in self.get_products('nandansons'):
      yield inv

  def get_rakuten_products(self):
    for inv in self.get_products('Rakuten'):
      yield inv

  def get_st11_products(self):
    for inv in self.get_products('11ST'):
      yield inv

  def get_apo_health_products(self):
    for inv in self.get_products('ApoHealth'):
      yield inv

  def get_lotteon_products(self):
    for inv in self.get_products('lotteon'):
      yield inv

  def get_wemakeprice_products(self):
    for inv in self.get_products('WEMAKEPRICE'):
      yield inv

  def get_naver_products(self):
    for inv in self.get_products('Naver'):
      yield inv

  def get_dangdang_products(self):
    for inv in self.get_products('Dangdang'):
      yield inv

  def get_coupang_products(self):
    for inv in self.get_products('Coupang'):
      yield inv

  def get_faire_products(self):
    for inv in self.get_products('faire'):
      yield inv

  def get_www_books_com_tw_products(self):
    for inv in self.get_products('www_books_com_tw'):
      yield inv

  def get_products(self, src, marketplace=None):
    source = None
    if src == 'Amazon' and marketplace:
      source = 'AMZ_{}'.format(marketplace.upper())
    elif src == 'Ebay' and marketplace:
      source = 'Ebay_{}'.format(marketplace.upper())
    elif src in [
        'Dangdang', 'Rakuten', 'Coupang', 'Naver', 'www_books_com_tw', "WEMAKEPRICE", "11ST",
        "nandansons", "lotteon", "ApoHealth", "EMP", "Boots", "Trendyol", "Daisomall"
        ]:
      source = src
    else:
      source = src

    if source is None:
      return

    # _, output_path = tempfile.mkstemp('.csv', '{}-'.format(source))
    temp_dir = tempfile.gettempdir()
    output_path = os.path.join(temp_dir, '{}-{}.csv'.format(source, str(uuid.uuid4())))
    try:
      logger.info("[InventoryPath] %s", output_path)
      self.spree_api.download_inventory(source, output_path)

      if not os.path.isfile(output_path):
        return

      try:
        manual_products = {str(pid):None for pid in self.get_manual_products()}
      except:
        manual_products = {}

      with open(output_path, encoding='utf-8', errors='ignore') as fh:
        reader = csv.reader(fh)
        headers = next(reader)
        headers_mapping = {
          'ProductID': 'product_id',
          'Source': 'source',
          'SourceProductID': 'source_product_id',
          'Handle': 'handle',
          'Variants': 'variants',
          'InStock': 'in_stock'
        }
        formatted_headers = []
        for header in headers:
          if header not in headers_mapping:
            continue

          formatted_headers.append(headers_mapping[header])

        for row in reader:
          if not row:
            continue

          record = dict()
          try:
            for idx, header in enumerate(formatted_headers):
              if header == 'variants':
                record[header] = json.loads(row[idx])
              else:
                record[header] = row[idx]
          except Exception as e:
            logger.info('[InventoryRow] %s', row)
            logger.exception(e)
            continue

          if not record['product_id'] or not record['source'] or not record['source_product_id']:
            continue

          if record['product_id'] in manual_products:
            continue

          yield record
    finally:
      if os.path.isfile(output_path):
        os.remove(output_path)

  def get_manual_products(self):
    result = None
    try:
      result = self.spree_api.get_manual_products()
    except Exception as e:
      logger.exception(e)

    if not result or 'error' in result or 'errors' in result:
      return []

    return result.get('products', [])

  def set_products_offer(self, prods, pool=None):
    store_offers = dict()
    for prod_id, prod in prods.items():
      store_offer = {'handle': prod['handle'], 'product_id': prod_id, 'offers': {}}
      variants = prod.get('variants', None)
      if not variants:
        continue

      offer = prod.get('offer', None)
      # Offer service encountered errors
      if isinstance(offer, bool) and not offer:
        continue

      # Product is not available on offer service
      if offer is None:
        continue

      # Product with multiple variants should supply each variant's offer
      if len(variants) > 1:
        if not isinstance(offer, dict):
          continue
      else:
        v = variants[0]
        if v['variant_id'] not in offer:
          offer = {v['variant_id']: offer}

      for variant in variants:
        v_offer = offer.get(variant['variant_id'], None)
        if not v_offer:
          continue

        price = round(float(v_offer['price']), 2)
        quantity = round(float(v_offer['quantity']), 2)
        target_offer = {
          'product_id': prod_id,
          'variant_id': variant['variant_id'],
          'price': price,
          'quantity': quantity,
        }
        src_offer = v_offer.get('src_offer', None)
        if src_offer:
          src_price = src_offer.get('price', None)
          src_currency = src_offer.get('currency', 'USD')
          if src_price:
            target_offer['cost_price'] = src_price
            target_offer['cost_currency'] = src_currency

        store_offer['offers'][variant['variant_id']] = target_offer
        logger.debug(
          '[InventoryUpdate] ProductId: %s, VariantId: %s, Price: %s, Quantity: %s',
          prod_id, variant['variant_id'], price, quantity)

      store_offers[prod_id] = store_offer

    resp = self.spree_api.set_offers(store_offers)
    logger.info('[InventoryUpdated] %s', resp)

    return store_offers

  def set_inventory(self, stock_items):
    succeed = True
    for stock_item in stock_items:
      max_retries = 3
      while max_retries > 0:
        try:
          resp = self.spree_api.update_stock(
            stock_item['id'], stock_item['quantity'], stock_item['stock_location_id'])
          if resp.status_code == 422:
            logger.debug(
              '[StockUpdateMsg] VariantId: %s, Stock: %s, Quantity: %s, Response: %s',
              stock_item['variant_id'], stock_item['id'], stock_item['quantity'], resp.text)
            max_retries -= 1
            time.sleep(1)

            continue

          try:
            stock = resp.json()
            if 'error' in stock or 'errors' in stock:
              succeed = False
              logger.warning(
                '[StockUpdateError] VariantId: %s, Stock: %s, Error: %s',
                stock_item['variant_id'], stock_item['id'],
                stock.get('error', stock.get('errors')))
            else:
              logger.info(
                '[StockUpdated] VariantId: %s, Stock: %s, Quantity: %s',
                stock_item['variant_id'], stock_item['id'], stock_item['quantity'])
          except Exception as e:
            succeed = False
            logger.exception(e)
            logger.warning('[StockUpdateInvalidResponse] %s', resp.text)

          break
        except Exception as e:
          logger.exception(e)
          max_retries -= 1
          time.sleep(1)

    return succeed

  def set_price(self, product_id, variant_id, price, cost=None):
    succeed = True
    max_retries = 3
    while max_retries > 0:
      try:
        if cost:
          resp = self.spree_api.update_variant(product_id, variant_id, price=price, cost_price=cost)
        else:
          resp = self.spree_api.update_variant(product_id, variant_id, price=price)
        if resp.status_code == 422:
          logger.debug(
            '[PriceUpdateMsg] ProductID: %s, VariantId: %s, Price: %s, Response: %s',
            product_id, variant_id, price, resp.text)
          max_retries -= 1
          time.sleep(1)

          continue

        sku = ''
        try:
          variant = resp.json()
          if 'error' in variant or 'errors' in variant:
            succeed = False
            logger.warning(
              '[PriceUpdateError] ProductId: %s, VariantId: %s, Error: %s',
              product_id, variant_id, variant.get('error', variant.get('errors')))
          elif 'sku' in variant and variant['sku']:
            sku = variant['sku']
            logger.info(
              '[PriceUpdated] ProductId: %s, VariantId: %s, Price: %s, SKU: %s',
              product_id, variant_id, price, sku)
          else:
            logger.info('[ResponseWithoutSKU] %s', resp.text)
        except:
          succeed = False
          logger.warning('[PriceUpdateInvalidResponse] %s', resp.text)

        break
      except Exception as e:
        succeed = False
        logger.warning(
          '[PriceUpdateError] ProductId: %s, VariantId: %s, Price: %s',
          product_id, variant_id, price)
        logger.exception(e)

    return succeed

  def download_inventory(self, source, output_path):
    self.spree_api.download_inventory(source, output_path)

    if not os.path.isfile(output_path):
      return

    with open(output_path, encoding='utf-8', errors='ignore') as fh:
      reader = csv.reader(fh)
      headers = next(reader)
      headers_mapping = {
        'ProductID': 'product_id',
        'Source': 'source',
        'SourceProductID': 'source_product_id',
        'Handle': 'handle',
        'Variants': 'variants'
      }
      formatted_headers = []
      for header in headers:
        if header not in headers_mapping:
          continue

        formatted_headers.append(headers_mapping[header])

      for row in reader:
        if not row:
          continue

        record = dict()
        try:
          for idx, header in enumerate(formatted_headers):
            if header == 'variants':
              record[header] = json.loads(row[idx])
            else:
              record[header] = row[idx]
        except Exception as e:
          logger.info('[InventoryRow] %s', row)
          logger.exception(e)
          continue

        yield record


if __name__ == "__main__":
  pass
